/*
 * @(#)BoardPanel.java				1.00
 * 
 * Copyright (C) Ramin Rakhamimov
 * ramin32@gmail.com
 * http://www.ramrak.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.ramrak.applications.thechessgame.board;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.util.Map;

import javax.swing.JPanel;

import net.ramrak.applications.thechessgame.eventhandlers.BoardKeyHandler;
import net.ramrak.applications.thechessgame.helpers.ChessImageToolKit;
import net.ramrak.applications.thechessgame.pieces.BoardPiece;
import net.ramrak.applications.thechessgame.pieces.PiecePosition;

/**
 * JPanel with a physical representation of the Chess Board.
 *
 * @author Ramin Rakhamimov
 */
public class BoardPanel extends JPanel {
	public final int SQUARES = 8; // per rows/cols
	private final int SQUARE_WIDTH;
	private final int DEFAULT_WIDTH;
	private final int DEFAULT_HEIGHT;

	private final BoardKeyHandler boardKeyHandler;

	private final Map<PiecePosition, BoardPiece> boardMap;
	private static final int THICKNESS = 3;

	public BoardPanel(Map<PiecePosition, BoardPiece> map) {
		boardMap = map;

		SQUARE_WIDTH = ChessImageToolKit.getSquareSize();

		DEFAULT_WIDTH = SQUARES * SQUARE_WIDTH;
		DEFAULT_HEIGHT = SQUARES * SQUARE_WIDTH;

		setPreferredSize(new Dimension(DEFAULT_WIDTH, DEFAULT_HEIGHT));
		boardKeyHandler = new BoardKeyHandler(this, boardMap);
		addKeyListener(boardKeyHandler);
	}

	@Override
	public void paintComponent(Graphics g) {
		drawSquares(g);
		drawPieces(g);

		drawCursor(g, boardKeyHandler.getCurrentPosition());
	}

	private void drawCursor(Graphics g, PiecePosition currentPosition) {
		g.setColor(Color.red);
		for (int i = 0; i < THICKNESS; i++) {
			Point p = currentPosition.getPoint(SQUARE_WIDTH);
			int x = (int) p.getX();
			int y = DEFAULT_HEIGHT - (int) p.getY() - SQUARE_WIDTH;
			g
					.drawRect(x + i, y + i, SQUARE_WIDTH - 2 * i, SQUARE_WIDTH
							- 2 * i);
		}

	}

	private void drawPieces(Graphics g) {
		for (PiecePosition position : boardMap.keySet()) {
			Point pos = position.getPoint(SQUARE_WIDTH);
			BoardPiece piece = boardMap.get(position);
			Image image = ChessImageToolKit.getPiece(piece.getType(), piece
					.getColor());
			drawImage(g, image, pos);
		}
	}

	private void drawImage(Graphics g, Image image, Point point) {
		g.drawImage(image, (int) point.getX(), DEFAULT_HEIGHT
				- (int) point.getY() - SQUARE_WIDTH, null);
	}

	private void drawSquares(Graphics g) {
		g.setColor(new Color(85, 107, 47));

		int squareWidth = getWidth() / SQUARES;
		int squareHeight = getHeight() / SQUARES;
		boolean drawTime = true;

		for (int row = 0; row < SQUARES; row++) {
			for (int col = 0; col < SQUARES; col++) {

				int x = col * squareWidth;
				int y = row * squareHeight;
				if (drawTime) {

					g.drawImage(ChessImageToolKit.DARK_SQUARE, x, y, null);
					drawTime = false;
				} else {
					g.drawImage(ChessImageToolKit.LIGHT_SQUARE, x, y, null);
					drawTime = true;
				}
				if (col == SQUARES - 1)
					drawTime = !drawTime;
			}
		}
	}

}
