/*
 * @(#)PiecePosition.java				1.00
 * 
 * Copyright (C) Ramin Rakhamimov
 * ramin32@gmail.com
 * http://www.ramrak.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.ramrak.applications.thechessgame.pieces;

import java.awt.Point;

/**
 * DataType to represent a piece position on the game board.
 * @author Ramin Rakhamimov
 */
public final class PiecePosition {
	private final int rank;
	private final int file;

	private PiecePosition(int file, int rank) {
		if (file < 0 || file > 7)
			throw new IllegalArgumentException("Wrong file: " + file);
		if (rank < 0 || rank > 7)
			throw new IllegalArgumentException("Wrong Row: " + rank);

		this.file = file;
		this.rank = rank;
	}

	public static PiecePosition createFromString(String str) {
		str = str.toUpperCase().trim();
		if (str.length() != 2) {
			throw new IllegalArgumentException(
					"String length must be of size 2: " + str);
		}
		return new PiecePosition(str.charAt(0) - 'A', str.charAt(1) - '1');
	}

	public static PiecePosition createFromFileRank(int file, int rank) {
		return new PiecePosition(file, rank);
	}

	public static PiecePosition createFromGraphicalPoint(Point p, int squareSize) {
		int tempFile = (int) (p.getX() / squareSize);
		int tempRank = (int) (p.getY() / squareSize);

		return new PiecePosition(tempFile, tempRank);
	}

	public PiecePosition left() {
		return createFromFileRank(file - 1, rank);
	}

	public PiecePosition right() {
		return createFromFileRank(file + 1, rank);

	}

	public PiecePosition up() {
		return createFromFileRank(file, rank + 1);
	}

	public PiecePosition down() {
		return createFromFileRank(file, rank - 1);
	}

	public int getRank() {
		return rank;
	}

	public int getFile() {
		return file;
	}

	public Point getPoint(int squareSize) {
		int x = file * squareSize;
		int y = rank * squareSize;
		return new Point(x, y);
	}

	public Point getCenteredPoint(int squareSize) {
		int x = file * squareSize;
		int y = rank * squareSize;
		int scale = squareSize / 2;
		return new Point(x + scale, y + scale);

	}

	public boolean equals(Object o) {
		if (this == o)
			return true;
		if (o == null || getClass() != o.getClass())
			return false;

		PiecePosition position = (PiecePosition) o;

		if (file != position.file)
			return false;
		if (rank != position.rank)
			return false;

		return true;
	}

	public int hashCode() {
		int result;
		result = rank;
		result = 31 * result + file;
		return result;
	}

	public String toString() {
		return "" + (char) (file + 'A') + (char) (rank + '1') + " Rank: "
				+ rank + " File: " + file;
	}
}
